using System;
using GameKnight.Core;

namespace GameKnight
{
	
	namespace GameBoy
	{

		public sealed partial class Cartridge : ICartridge
		{

				public sealed class MBC1 : Memory
				{
	                 
					private enum Modes
					{
						SixteenEight = 0,
						FourThirtyTwo = 1,
					}
	                 			
				    private Modes Mode = Modes.SixteenEight;
	
					public MBC1( IConsole Console, Cartridge Cartridge ) : base( Console, Cartridge )
					{
						return;
					}
	
					public override void WriteU8( int Address, byte Value )
					{
						switch( Address & 0xE000 )
						{
							case 0x0000:
								RAM_Enabled = ( ( Value & 0x0A ) == 0x0A );
								break;
							case 0x2000:
								ROM_BankIndex &= 0x60;
								ROM_BankIndex |= ( Value & 0x1F );
								ROM_BankIndex %= ROM.Length;
								ROM_BankIndex = ( ( ROM_BankIndex != 0 ) ? ROM_BankIndex : 1 );
								break;
							case 0x4000:
								if( Mode == Modes.SixteenEight )
								{
									ROM_BankIndex &= 0x1F;
									ROM_BankIndex |= ( ( Value & 0x03 ) << 5 );
									ROM_BankIndex %= ROM.Length;
									ROM_BankIndex = ( ( ROM_BankIndex != 0 ) ? ROM_BankIndex : 1 );
								}
								else
								{
									RAM_BankIndex = ( Value & 0x03 );
								}
								break;
							case 0x6000:
								Mode = (Modes)( Value & 0x01 );
								break;
							default:
								base.WriteU8( Address, Value );
								break;
						}
						return;
					}
				
				}
			
		}
		
	}
	
}


