﻿using System;
using IO = System.IO;
using GameKnight.Core;

namespace GameKnight
{
	
	namespace GameBoy
	{
	
		public sealed partial class Cartridge : ICartridge
		{

			private abstract partial class Mapper : Memory
			{
				
				protected Bank[] RAM = null;
				
				protected int RAM_BankIndex = 0;
	
				protected bool RAM_Enabled = false;
	
				protected Bank[] ROM = null;
						    
				protected int ROM_BankIndex = 1;
				    
				protected Mapper( IConsole Console, Cartridge Cartridge ) : base( Console )
				{
					this.ROM = Cartridge.ROM;
					this.RAM = Cartridge.RAM;
					return;
				}

				public override bool Load( IO.BinaryReader BinaryReader )
				{
					if( BinaryReader.ReadString() != "Mapper" )
	            	{
	            		return false;
	            	}
					if( BinaryReader.ReadInt32() != 0 )
	            	{
	            		return false;
	            	}			
					RAM_BankIndex = BinaryReader.ReadInt32();
					RAM_Enabled = BinaryReader.ReadBoolean();
					ROM_BankIndex = BinaryReader.ReadInt32();
					if( RAM != null )
					{
						for( int i = 0; i < RAM.Length; i++ )
						{
							RAM[i].Load( BinaryReader );
						}
					}
					return base.Load( BinaryReader );
				}
				
				public override byte ReadU8( int Address )
				{
					switch( Address & 0xE000 )
					{
						case 0x0000:
						case 0x2000:
							return ROM[0][Address];
						case 0x4000:
						case 0x6000:
							return ROM[ROM_BankIndex][Address];
						case 0xA000:
							if( ( RAM != null ) && ( RAM_Enabled ) )
							{
								return RAM[RAM_BankIndex][Address];
							}
							break;
						default:
                            return base.ReadU8( Address );
					}
					return 0;
				}
				
				public override bool Save( IO.BinaryWriter BinaryWriter )
				{
					BinaryWriter.Write( "Mapper" );
					BinaryWriter.Write( ((int)0) );
					BinaryWriter.Write( RAM_BankIndex );
					BinaryWriter.Write( RAM_Enabled );
					BinaryWriter.Write( ROM_BankIndex );
					if( RAM != null )
					{
						for( int i = 0; i < RAM.Length; i++ )
						{
							RAM[i].Save( BinaryWriter );
						}
					}
					return base.Save( BinaryWriter );
				}
	
				public override void WriteU8( int Address, byte Value )
				{
					switch( Address & 0xE000 )
					{
						case 0x0000:
						case 0x2000:
						case 0x4000:
						case 0x6000:
							break;
						case 0xA000:
							if( ( RAM != null ) && ( RAM_Enabled ) )
							{
								RAM[RAM_BankIndex][Address] = Value;
							}
							break;
						default:
							base.WriteU8( Address, Value );
							break;
					}
					return;
				}
					
			}
			
		}

	}
	
}
