﻿using System;
using GameKnight.Core;

namespace GameKnight
{
	
	namespace GameBoy
	{
	
		public partial class Processor : IProcessor
		{
			
			protected partial class Opcode
			{
				
				public static int RL( Processor Processor, Opcode Opcode )
				{
					int RegisterIndex = ( Opcode.Value & 0x07 );
					byte Value = Processor.GetRegisterValue( RegisterIndex );
					int Carry = ( Value & 0x80 );
					Value = (byte)( ( Value << 1 ) | ( Processor.CarryFlag ? 1 : 0 ) );
					Processor.ClearFlags();
					Processor.CarryFlag = ( Carry != 0 );
					Processor.ZeroFlag = ( Value == 0 );
					Processor.SetRegisterValue( RegisterIndex, Value );
					return 0;
				}
				
				public static int RLA( Processor Processor, Opcode Opcode )
				{
					int Carry = ( Processor.A & 0x80 );
					Processor.A = (byte)( ( Processor.A << 1 ) | ( Processor.CarryFlag ? 1 : 0 ) );
					Processor.NegativeFlag = false;
					Processor.HalfCarryFlag = false;
					Processor.CarryFlag = ( Carry != 0 );
					return 0;
				}

				public static int RLC( Processor Processor, Opcode Opcode )
				{
					int RegisterIndex = ( Opcode.Value & 0x07 );
					byte Value = Processor.GetRegisterValue( RegisterIndex );					
					Value = (byte)( ( Value << 1 ) | ( Value >> 7 ) );
					Processor.ClearFlags();
					Processor.CarryFlag = ( ( Value & 0x01 ) != 0 );
					Processor.ZeroFlag = ( Value == 0 );
					Processor.SetRegisterValue( RegisterIndex, Value );
					return 0;
				}
				
				public static int RLCA( Processor Processor, Opcode Opcode )
				{
					Processor.A = (byte)( ( Processor.A << 1 ) | ( Processor.A >> 7 ) );
					Processor.NegativeFlag = false;
					Processor.HalfCarryFlag = false;
					Processor.CarryFlag = ( ( Processor.A & 0x01 ) != 0 );
					return 0;
				}
				
				public static int RR( Processor Processor, Opcode Opcode )
				{
					int RegisterIndex = ( Opcode.Value & 0x07 );
					byte Value = Processor.GetRegisterValue( RegisterIndex );					
					int Carry = ( Value & 0x01 );				
					Value = (byte)( ( Value >> 1 ) | ( Processor.CarryFlag ? 0x80 : 0 ) );				
					Processor.ClearFlags();
					Processor.CarryFlag = ( Carry != 0 );	
					Processor.ZeroFlag = ( Value == 0 );
					Processor.SetRegisterValue( RegisterIndex, Value );
					return 0;
				}
				
				public static int RRA( Processor Processor, Opcode Opcode )
				{
					int Carry = ( Processor.A & 0x01 );
					Processor.A = (byte)( ( Processor.A >> 1 ) | ( Processor.CarryFlag ? 0x80 : 0 ) );
					Processor.NegativeFlag = false;
					Processor.HalfCarryFlag = false;
					Processor.CarryFlag = ( Carry != 0 );
					return 0;
				}
				
				public static int RRC( Processor Processor, Opcode Opcode )
				{
					int RegisterIndex = ( Opcode.Value & 0x07 );
					byte Value = Processor.GetRegisterValue( RegisterIndex );					
					Value = (byte)( ( Value >> 1 ) | ( Value << 7 ) );				
					Processor.ClearFlags();
					Processor.CarryFlag = ( ( Value & 0x80 ) != 0 );
					Processor.ZeroFlag = ( Value == 0 );
					Processor.SetRegisterValue( RegisterIndex, Value );
					return 0;
				}
				
				public static int RRCA( Processor Processor, Opcode Opcode )
				{
					Processor.A = (byte)( ( Processor.A >> 1 ) | ( Processor.A << 7 ) );	
					Processor.NegativeFlag = false;
					Processor.HalfCarryFlag = false;
					Processor.CarryFlag = ( ( Processor.A & 0x80 ) != 0 );
					return 0;
				}
				
				public static int SLA( Processor Processor, Opcode Opcode )
				{
					int RegisterIndex = ( Opcode.Value & 0x07 );
					byte Value = Processor.GetRegisterValue( RegisterIndex );										
					Processor.ClearFlags();
					Processor.CarryFlag = ( ( Value & 0x80 ) != 0 );
					Value <<= 1;
					Processor.ZeroFlag = ( Value == 0 );
					Processor.SetRegisterValue( RegisterIndex, Value );
					return 0;
				}
				
				public static int SRA( Processor Processor, Opcode Opcode )
				{
					int RegisterIndex = ( Opcode.Value & 0x07 );
					byte Value = Processor.GetRegisterValue( RegisterIndex );										
					Processor.ClearFlags();
					Processor.CarryFlag = ( ( Value & 0x01 ) != 0 );
					Value = (byte)( ( Value >> 1 ) | ( Value & 0x80 ) );
					Processor.ZeroFlag = ( Value == 0 );
					Processor.SetRegisterValue( RegisterIndex, Value );
					return 0;
				}
				
				public static int SRL( Processor Processor, Opcode Opcode )
				{
					int RegisterIndex = ( Opcode.Value & 0x07 );
					byte Value = Processor.GetRegisterValue( RegisterIndex );												
					Processor.ClearFlags();
					Processor.CarryFlag = ( ( Value & 0x01 ) != 0 );
					Value >>= 1;
					Processor.ZeroFlag = ( Value == 0 );
					Processor.SetRegisterValue( RegisterIndex, Value );
					return 0;
				}
				
				public static int SWAP( Processor Processor, Opcode Opcode )
				{
					int RegisterIndex = ( Opcode.Value & 0x07 );
					byte Value = Processor.GetRegisterValue( RegisterIndex );				
					Value = (byte)( ( ( Value & 0xF0 ) >> 4 ) | ( ( Value & 0x0F ) << 4 ) );
					Processor.ClearFlags();
					Processor.ZeroFlag = ( Value == 0 );
					Processor.SetRegisterValue( RegisterIndex, Value );
					return 0;
				}
				
			}
			
		}
		
	}
	
}
	
