﻿using System;
using GameKnight.Core;

namespace GameKnight
{
	
	namespace GameBoy
	{
	
		public partial class Register : IRegister
		{

			public class LCDC : IRegister
			{

				public bool BackgroundEnabled
				{
					get
					{
						return ( ( DataArray[0] & 0x01 ) != 0 );
					}
				}
				
				public bool Coincidence
				{
					get
					{
						return ( ( DataArray[1] & 0x40 ) != 0 );
					}
				}
				
				public bool CoincidenceFlag
				{
					get
					{
						return ( ( DataArray[1] & 0x04 ) != 0 );
					}
					set
					{
						DataArray[1] = (byte)( value ? ( DataArray[1] | 0x04 ) : ( DataArray[1] & ~0x04 ) );
					}
				}

				private readonly IConsole Console = null;

				private byte[] DataArray = new byte[6] { 0x91, 0x00 , 0x00, 0x00, 0x00, 0x00 };

				public bool DisplayEnabled
				{
					get
					{
						return ( ( DataArray[0] & 0x80 ) != 0 );
					}
				}
				
				private readonly Register.IE IE = null;

				public int InterruptSelection
				{
					get
					{
						return ( ( DataArray[1] >> 3 ) & 0x07 );
					}
				}
								
				public int Map
				{
					get
					{
						return ( ( DataArray[0] >> 3 ) & 0x01 );
					}
				}
				
				public int ModeFlag
				{
					get
					{
						return ( DataArray[1] & 0x03 );
					}
					set
					{
						DataArray[1] &= 0xFC;
						DataArray[1] |= (byte)( value & 0x03 );
						if( DisplayEnabled )   
						{
							if( ModeFlag == 1 )
							{
								Console.InterruptMask |= Interrupts.VideoBlank;
							}
							else if( ( ( InterruptSelection & ( 1 << ModeFlag ) ) != 0 ) && ( CoincidenceFlag == false ) )
							{
			              	 	Console.InterruptMask |= Interrupts.STAT;
							}
							if( ModeFlag == 3 )
							{
								CheckScanLineInterrupt();
							}
						}
					}
				}
				
				public int PatternTable
				{
					get
					{
						return ( ( ( DataArray[0] & 0x10 ) == 0 ) ? 1 : 0 );
					}
				}
				
				public int ScanLine
				{
					get
					{
						return DataArray[4];
					}
					set
					{
                   		DataArray[4] = (byte)( value % 154 );
					}
				}
			
				public bool SpritesEnabled
				{
					get
					{
						return ( ( DataArray[0] & 0x02 ) != 0 );
					}
				}
				
				public int SpriteHeight
				{
					get
					{
						return ( ( ( DataArray[0] & 0x04 ) == 0 ) ? 8 : 16 );
					}
				}
	
				public int Window
				{
					get
					{
						return ( ( DataArray[0] >> 6 ) & 1 );
					}
				}

				public bool WindowEnabled
				{
					get
					{
						return ( ( DataArray[0] & 0x20 ) != 0 );
					}
				}

				public LCDC( IConsole Console )
				{
					this.Console = Console;
					this.IE = Console.GetRegister<Register.IE>();
					return;
				}
				
				public byte this[int Address]
				{
					get
					{
						switch( Address & 0x07 )
						{
							case 1:
								return (byte)( 0x80 | DataArray[1] );
							default:
								return DataArray[Address&0x07];
						}
					}
					set
					{
						switch( Address & 0x07 )
						{
							case 1:
								DataArray[1] = (byte)( ( value & 0xF8 ) | ( DataArray[1] & 0x07 ) );
								if( ( DisplayEnabled ) && ( ModeFlag <= 1 ) )
								{
									Console.InterruptMask |= Interrupts.STAT;
								}
								break;
							case 5:
                                DataArray[5] = value;
                                CheckScanLineInterrupt();
                                break;
							default:
								DataArray[Address&0x07] = value;
								break;
						}
					}
				}

				private void CheckScanLineInterrupt()
				{
					CoincidenceFlag = ( DataArray[4] == DataArray[5] );
					if( ( DisplayEnabled ) && ( CoincidenceFlag ) )
					{
						if( ( Coincidence ) && ( ( IE.Value & Interrupts.STAT ) != 0 ) )
						{
							Console.InterruptMask |= Interrupts.STAT;
						}
					}
					return;
				}
				
			}
			
		}
		
	}
	
}
