using System;
using GameKnight.Core;

namespace GameKnight
{

	namespace GameBoy
	{

		public partial class Register : IRegister
		{

			public sealed class P1 : Register, IController
			{

				public IConsole Console
				{
					get
					{
						return ConsoleProperty;	
					}
					private set
					{
						ConsoleProperty = value;
					}
				}
				
				private IConsole ConsoleProperty = null;
			
				public IControl[] ControlArray
				{
					get
					{
						return ControlArrayProperty;	
					}
					private set
					{
						ControlArrayProperty = value;
					}
				}
				
				private IControl[] ControlArrayProperty = null;
				
				private readonly byte[] DataArray = new byte[2];
			
				public bool Enabled
				{
					get
					{
						return true;
					}
					set
					{
						return;
					}
				}

				public string Name
				{
					get
					{
						return "Player 1";
					}
				}
					
				public P1( IConsole Console ) : base()
				{
					this.Console = Console;
					this.ControlArray = new IControl[]
					{
						new Button( this, 0x01, "Button A" ),
						new Button( this, 0x02, "Button B" ),
						new Button( this, 0x04, "Select" ),
						new Button( this, 0x08, "Start" ),
						new Button( this, 0x11, "Right" ),
						new Button( this, 0x12, "Left" ),
						new Button( this, 0x14, "Up" ),
						new Button( this, 0x18, "Down" ),
					};
					return;
				}
				
				public override byte this[int Address]
				{
					get
					{
						if( ( Value == 0x10 ) || ( Value == 0x20 ) )
						{
							return (byte)( Value | ( (~DataArray[(Value>>4)-1]) & 0x0F ) );
						}
						return (byte)( Value | 0x0F );
					}
					set
					{
						Value = (byte)( ( Value & 0xCF ) | ( value & 0x30 ) );
					}
				}
				
				public int this[IControl Control]
				{
					get
					{
						int Index = ( ((Button)Control).Mask >> 4 );
						int Mask = ( ((Button)Control).Mask & 0x0F );
						return Convert.ToInt32( ( ( DataArray[Index] & Mask ) != 0 ) );
					}
					set
					{
						int Index = ( ((Button)Control).Mask >> 4 );
						int Mask = ( ((Button)Control).Mask & 0x0F );
						if( value != 0 )
						{
							if( ( DataArray[Index] & Mask ) == 0 )
							{
								Console.InterruptMask |= Interrupts.ButtonHighToLow;
							}
							DataArray[Index] |= (byte)Mask;
						}
						else
						{
							DataArray[Index] &= (byte)( ~Mask );
						}
					}
				}

			}

		}

	}

}




