using System;
using GameKnight.Core;

namespace GameKnight
{
	
	namespace GameBoy
	{

		public partial class Register : IRegister
		{
			
			public class TMR : IRegister
			{
                
				private bool Active
				{
					get
					{
						return ( ( DataArray[3] & 0x04 ) != 0 );
					}
				}

				private readonly IConsole Console = null;

				private double Count = 0.0;
            
				private static readonly double[] CycleArray = new  double[]
            	{
					4096.0,
            		262144.0,
            		65536.0,
            		16384.0,
            	};

				private double Cycles = CycleArray[0];

				private readonly byte[] DataArray = new byte[4];

				//private double LastTimePassed = 0.0;
				
				private double LastCount = 0.0;
				
				private int Ticks = 0;
				
				private readonly ITimer Timer = null;

				public TMR( IConsole Console )
				{
					this.Console = Console;
					this.Timer = Console.Interface.Timer;
					return;
				}
				
				public byte this[int Address]
				{
					get
					{
						switch( Address & 0x03 )
						{
							case 0:
								DataArray[0] = (byte)( 16384.0 * Timer.GetInterval( this ) );
								return DataArray[0];
							case 1:
								Update();
								goto default;
							case 3:
								return (byte)( 0xF8 | DataArray[3] );
							default:
								return DataArray[Address&0x03];
						}
					}
					set
					{
                    	switch( Address & 0x03 )
						{
							case 0:
								DataArray[0] = 0;
								Timer.GetInterval( this );
								break;
							case 2:
								Cycles = CycleArray[value&0x03];
								goto default;
							default:
								DataArray[Address&0x03] = value;
								break;
						}
					}
				}

				public void Update()
				{
					if( Active )
					{
						Count = ( CycleArray[DataArray[3]&0x03] * Timer.TimePassed );
						if( ((int)( Count - LastCount )) > 0 )
						{
							Ticks += (int)( Count - LastCount );
		                    LastCount = Count;
		                    if( Ticks >= 0x100 )
		                    {
		                    	Ticks = ( DataArray[2] + ( Ticks & 0xFF ) );
								Console.InterruptMask |= Interrupts.TimerOverflow;
							}
						}
	                }
					DataArray[1] = (byte)Ticks;
					/*if( Active )
					{
						Cycles -= Application.TimePassed;
	                    while( Cycles <= 0.0 )
	                    {
	                    	DataArray[1]++;
	                    	if( DataArray[1] == 0 )
	                    	{
	                    		DataArray[1] = DataArray[2];
								Console.InterruptMask |= Interrupts.TimerOverflow;
	                    	}
	                    	Cycles += CycleArray[DataArray[3]&0x03];
						}
	                }*/			
					return;
				}

			}

		}

	}
	
}
