﻿using System;
using Collections = System.Collections.Generic;
using IO = System.IO;
using GameKnight.Core;

namespace GameKnight.XNA
{

	public sealed class Options : IOptions
	{
		
		public event EventHandler ValueChanged = null;

		private readonly Collections.Dictionary<string,Collections.Dictionary<string,string>> ValueList = new Collections.Dictionary<string,Collections.Dictionary<string,string>>();
		
		public Options()
		{
			ValueList.Add( "VideoProcessor", new Collections.Dictionary<string,string>() );
			ValueList["VideoProcessor"].Add( "FrameSkip", "0" );
			return;
		}
		
		public void Get( string Key, string Name, ref string Value )
		{
			string Result = Read( Key, Name );
			if( Result != "" )
			{
				Value = Read( Key, Name );
			}
			return;
		}
		
		public void Get( string Key, string Name, ref int Value )
		{
			int Result = 0;
			if( int.TryParse( Read( Key, Name ), out Result ) )
			{
				Value = Result;
			}
			return;
		}
		
		public void Get( string Key, string Name, ref double Value )
		{
			double Result = 0;
			if( double.TryParse( Read( Key, Name ), out Result ) )
			{
				Value = Result;
			}
			return;
		}
		
		public void Get<T>( string Key, string Name, ref T Value )
		{
			if( typeof(T).BaseType != typeof(Enum) )
			{
				return;
			}
			object Result = null;
			try
			{
				Result = Enum.Parse( typeof( T ), Read( Key, Name ), true );
			}
			catch
			{
				return;
			}
			Value = (T)Result;
			return;
		}
		
		private string Read( string Key, string Name )
		{
			if( ValueList.ContainsKey( Key ) )
			{
				if( ValueList[Key].ContainsKey( Name ) )
				{
					return ValueList[Key][Name];
				}
			}
			return "";
		}
		
		public void Set( string Key, string Name, string Value )
		{
			Write( Key, Name, Value );
			return;
		}
		
		public void Set( string Key, string Name, int Value )
		{
			Write( Key, Name, Value );
			return;
		}
		
		public void Set<T>( string Key, string Name, T Value )
		{
			if( typeof(T).BaseType != typeof(Enum) )
			{
				return;
			}
			Write( Key, Name, Value );
			return;
		}
		
		private void Write( string Key, string Name, object Value )
		{
			if( ( Key == null ) || ( Name == null ) || ( Value == null ) )
			{
				return;
			}
			if( ValueList.ContainsKey( Key ) == false )
			{
				if( ( Key == "root" ) || ( Key == "" ) )
				{
					return;
				}
				ValueList.Add( Key, new Collections.Dictionary<string,string>() );
			}
			if( ValueList[Key].ContainsKey( Name ) )
			{
				ValueList[Key][Name] = Value.ToString();
			}
			else if( Name != "" )
			{
				ValueList[Key].Add( Name, Value.ToString() );
			}
			if( ValueChanged != null )
			{
				ValueChanged( this, EventArgs.Empty );
			}
			return;
		}
		
	}
	
}
